#
# Copyright (c) 2020 Kian Cross
#

from typing import cast
from pixy.escape_sequence import EscapeSequence
from pixy.colours import constants


class TrueColour(EscapeSequence):

    __MAX_VALUE = 0xFF
    __RGB_SUB_CODE = 2

    def __new__(
        cls, red: int, green: int, blue: int, background: bool = False
    ) -> "TrueColour":

        if min(red, green, blue) < 0 or max(red, green, blue) > cls.__MAX_VALUE:

            raise ValueError("RGB values must be in-between 0 and %d" % cls.__MAX_VALUE)

        if background:
            type_code = constants.BACKGROUND_CODE
        else:
            type_code = constants.FOREGROUND_CODE

        return cast(
            TrueColour,
            super().__new__(cls, type_code, cls.__RGB_SUB_CODE, red, green, blue),
        )



#
# Copyright (c) 2020 Kian Cross
#

from pixy.pring import pring
from pixy.colours import foreground, background
from pixy import decorators
from pixy.font import Font
from pixy.escape_sequence import EscapeSequence
from pixy.extended_colour import ExtendedColour
from pixy.true_colour import TrueColour
from pixy.helpers import black, red, green, yellow, blue, magenta, cyan, white

__all__ = [
    "pring",
    "foreground",
    "background",
    "decorators",
    "Font",
    "EscapeSequence",
    "ExtendedColour",
    "TrueColour",
    "black",
    "red",
    "green",
    "yellow",
    "blue",
    "magenta",
    "cyan",
    "white",
]



#
# Copyright (c) 2020 Kian Cross
#

from typing import cast
from pixy.escape_sequence import EscapeSequence
from pixy.colours import constants


class ExtendedColour(EscapeSequence):
    def __new__(cls, colour_code: int, background: bool = False) -> "ExtendedColour":

        if colour_code < 0 or colour_code > 255:
            raise ValueError("colour_code must be in-between 0 and 255")

        if background:
            type_code = constants.BACKGROUND_CODE
        else:
            type_code = constants.FOREGROUND_CODE

        return cast(ExtendedColour, super().__new__(cls, type_code, 5, colour_code))



#
# Copyright (c) 2020 Kian Cross
#

from typing import cast
from pixy.escape_sequence import EscapeSequence


class Font(EscapeSequence):

    __FONT_START_CODE = 11
    __FONT_END_CODE = 19

    def __new__(cls, font_number: int) -> "Font":

        number_of_fonts = cls.__FONT_END_CODE - cls.__FONT_START_CODE

        if font_number < 0 or font_number > number_of_fonts:
            raise ValueError("font must be in-between 0 and %d" % number_of_fonts)

        return cast(Font, super().__new__(cls, cls.__FONT_START_CODE + font_number))



#
# Copyright (c) 2020 Kian Cross
#

from pixy.escape_sequence import EscapeSequence
from pixy.colours import constants

black = EscapeSequence(constants.FOREGROUND_START_CODE + constants.BLACK)
red = EscapeSequence(constants.FOREGROUND_START_CODE + constants.RED)
green = EscapeSequence(constants.FOREGROUND_START_CODE + constants.GREEN)
yellow = EscapeSequence(constants.FOREGROUND_START_CODE + constants.YELLOW)
blue = EscapeSequence(constants.FOREGROUND_START_CODE + constants.BLUE)
magenta = EscapeSequence(constants.FOREGROUND_START_CODE + constants.MAGENTA)
cyan = EscapeSequence(constants.FOREGROUND_START_CODE + constants.CYAN)
white = EscapeSequence(constants.FOREGROUND_START_CODE + constants.WHITE)



#
# Copyright (c) 2020 Kian Cross
#

from pixy.pring import pring
from pixy.colours import foreground


def black(text: str) -> str:
    return pring(text, foreground.black)


def red(text: str) -> str:
    return pring(text, foreground.red)


def green(text: str) -> str:
    return pring(text, foreground.green)


def yellow(text: str) -> str:
    return pring(text, foreground.yellow)


def blue(text: str) -> str:
    return pring(text, foreground.blue)


def magenta(text: str) -> str:
    return pring(text, foreground.magenta)


def cyan(text: str) -> str:
    return pring(text, foreground.cyan)


def white(text: str) -> str:
    return pring(text, foreground.white)



#
# Copyright (c) 2020 Kian Cross
#

from pixy.escape_sequence import EscapeSequence


reset = EscapeSequence(0)


def pring(text: str, style: EscapeSequence = EscapeSequence()) -> str:

    if not isinstance(text, str):
        raise ValueError("text must be of type str")

    if not isinstance(style, EscapeSequence):
        raise ValueError("style must be of type EscapeSequence (or sub-class of).")

    sections = [section for section in text.split(reset.code) if section]

    return style.code + (reset.code + style.code).join(sections) + reset.code



#
# Copyright (c) 2020 Kian Cross
#

BLACK = 0
RED = 1
GREEN = 2
YELLOW = 3
BLUE = 4
MAGENTA = 5
CYAN = 6
WHITE = 7

FOREGROUND_START_CODE = 30
BACKGROUND_START_CODE = 40

FOREGROUND_CODE = 38
BACKGROUND_CODE = 48



#
# Copyright (c) 2020 Kian Cross
#

from typing import Tuple, Any


class EscapeSequence(Tuple[int]):

    __ESCAPE_CHARACTER = "\x1b"

    def __new__(cls, *codes: int) -> "EscapeSequence":

        for code in codes:
            if not isinstance(code, int):
                raise TypeError("All codes must be integers.")

        return super(EscapeSequence, cls).__new__(cls, codes)  # type: ignore

    def __add__(self, other: Any) -> "EscapeSequence":
        return EscapeSequence(*super().__add__(other))

    def __str__(self) -> str:
        return self.code

    @property
    def code(self) -> str:
        return "%s[%sm" % (
            self.__ESCAPE_CHARACTER,
            ";".join(str(code) for code in self),
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pixy', 'pixy.colours']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pixy',
    'version': '1.0.2',
    'description': 'Add colour and style to terminal text',
    'long_description': 'pixy\n====\n\n[![PyPI version](https://badge.fury.io/py/pixy.svg)](https://pypi.org/project/pixy/)\n![Continuous Integration](https://github.com/kiancross/pixy/workflows/Continuous%20Integration/badge.svg)\n[![codecov](https://codecov.io/gh/kiancross/pixy/branch/master/graph/badge.svg?token=P490PT52VR)](https://codecov.io/gh/kiancross/pixy)\n\npixy is a Python library for adding colour and style to terminal text.\n\n* [Getting Started](#getting-started)\n  * [Installation](#installation)\n  * [Usage](#usage)\n* [Documentation](#documentation)\n* [Examples](#examples)\n* [Tests](#tests)\n* [License](#license)\n\nTerminal output can be styled using [ANSII escape code][ansii_escape_codes]. pixy provides a simple to use yet comprehensive wrapper, abstracting away the complexities.\n\npixy supports:\n  - 3-bit, 4-bit and 8-bit colour;\n  - 24-bit true colour RGB;\n  - changing the foreground and background colour;\n  - text decorators e.g. blink, underline, bold, italic, strike-through;\n  - custom fonts for compliant terminals;\n  - custom ANSII codes for development on non-spec-compliant terminals.\n\n[ansii_escape_codes]: https://en.wikipedia.org/wiki/ANSI_escape_codes\n\nGetting Started\n---------------\n\n### Installation\n\npixy can be installed using `pip`.\n```\n$ python3 -m pip install pixy\n```\n\n### Usage\n\npixy lets you style text straight out of the box.\n\n```python\nimport pixy\n\n# Print "Hello World" in red.\nprint(pixy.red("Hello World"))\n```\n\nIt provides the flexibility to format text as you wish.\n\n```python\nimport pixy\n\n# Create a string "Hello" with bold text and blue background.\ns1 = pixy.pring("Hello", pixy.decorators.bold + pixy.background.blue)\n\n# Concatenate the two strings, giving everything a red background (the\n# red background won\'t be applied to "Hello" because we\'ve already\n# given it a blue background!)\nprint(pixy.pring(s1 + " World. Lorem ipsum dolor...", pixy.background.red))\n```\n\nDocumentation\n-------------\nNot all terminals support all ANSII escape codes. Please check the terminal you are testing on supports the features you are using before opening an issue.\n\n### Helpers \nYou can create a string of a certain colour using the helper functions below:\n\n  - `pixy.black(text)`\n  - `pixy.red(text)`\n  - `pixy.green(text)`\n  - `pixy.yellow(text)`\n  - `pixy.blue(text)`\n  - `pixy.magenta(text)`\n  - `pixy.cyan(text)`\n  - `pixy.white(text)`\n\n#### Example\n```python\nimport pixy\n\n# Print "Hello World" in red.\nprint(pixy.red("Hello World"))\n```\n\n### `pixy.pring(text, style)`\n`pixy.pring` allows you to generate a string with any of the styles documented below. You can use more than one style by adding them together.\n\n  - `text` - the text to apply the style to.\n  - `style` - the style to apply to the text.\n\n#### Example\n```python\nimport pixy\n\n# Print "Hello World" with green text and white background.\nprint(pixy.pring(\n\t"Hello World",\n\tpixy.foreground.green + pixy.background.white\n))\n```\n\n### `pixy.foreground`, `pixy.background`\n`pixy.foreground` and `pixy.background` contain the variables: `black`,\n`red`, `green`, `yellow`, `blue`, `magenta`, `cyan`, `white`. \n\nUsing the `pixy.foreground` variant of the colour will apply the colour\nto the foreground and the `pixy.background` variant will apply it to the\nbackground.\n\nThese colours are known as the 3-bit colours and almost all terminals support\nthem. Most terminals also let you make them bright by adding the\n`pixy.decorators.bold` style.\n\n#### Example\n```python\nimport pixy\n\n# Print "Hello World" with green text.\nprint(pixy.pring("Hello World", pixy.foreground.green))\n```\n\n### `pixy.decorators`\nDecorators let you apply other styles to the text.\n\n| Decorator | Description | Support |\n|--|--|--|\n| `pixy.decorators.bold` | Bold/bright |\n| `pixy.decorators.faint` | Faint/dimmed |\n| `pixy.decorators.italic` | Italicised |\n| `pixy.decorators.underline` | Underlined |\n| `pixy.decorators.slow_blink` | Slow blink (less than 150 per minute) |\n| `pixy.decorators.rapid_blink` | Rapid blink (150+ per minute) | Not widely supported |\n| `pixy.decorators.invert` | Swap foreground and background colours |\n| `pixy.decorators.conceal` | Hide | Not widely supported |\n| `pixy.decorators.strike` | Strike-through |\n| `pixy.decorators.fraktur` | [Fraktur](https://en.wikipedia.org/wiki/Fraktur) | Rarely supported |\n| `pixy.decorators.double_underline` | Double underline |\n| `pixy.decorators.framed` | | Not widely supported |\n| `pixy.decorators.encircled` |  | Not widely supported |\n| `pixy.decorators.overlined` | Overlined |\n\n(Descriptions adapted from [Wikipedia](https://en.wikipedia.org/wiki/ANSI_escape_code#SGR_parameters))\n\n#### Example\n```python\nimport pixy\n\n# "Hello World" underlined.\nprint(pixy.pring("Hello World", pixy.decorators.underline))\n```\n\n### `pixy.ExtendedColour(code, background=False)`\nWith `pixy.ExtendedColour` you can select from a pre-defined selection of 256\ncolours.\n\n  - `code` - corresponds to a colour code found\n     [here](https://en.wikipedia.org/wiki/ANSI_escape_code#8-bit). \n\n  - `background` - boolean value indicating if this is a background or\n     foreground colour.\n\n#### Example\n```python\nimport pixy\n\n# Print "Hello World" in violet.\nprint(pixy.pring("Hello World", pixy.ExtendedColour(99)))\n\n# Print "Hello World" with a violet background.\nprint(pixy.pring("Hello World", pixy.ExtendedColour(99, True)))\n```\n\n### `pixy.TrueColour(red, green, blue, background=False)`\nWith `pixy.TrueColour` you can create an RGB colour.\n\n  - `red` - value between `0` and `255` indicating the intensity of the\n    red component.\n\n  - `green` - value between `0` and `255` indicating the intensity of the\n    green component.\n\n  - `blue` - value between `0` and `255` indicating the intensity of the\n    blue component.\n\n  - `background` - boolean value indicating if this is a background  or\n    foreground colour.\n\n#### Example\n```python\nimport pixy\n\n# Print "Hello World" in violet.\nprint(pixy.pring("Hello World", pixy.TrueColour(238, 130, 238)))\n\n# Print "Hello World" with a violet background.\nprint(pixy.pring("Hello World", pixy.TrueColour(238, 130, 238, True)))\n```\n\n### `pixy.Font(code)`\nAllows you to change the font used. Most terminals do not support this.\n\n  - `code` - a number between `0` and `8` corresponding to a font.\n\n#### Example\n```python\nimport pixy\n\nprint(pixy.pring("Hello World", pixy.Font(3)))\n```\n\n### `pixy.EscapeSequence(...)`\n`pixy.EscapeSequence` can be used in instances that you want to defined your own ANSII escape sequence.\n\n`pixy.EscapeSequence` takes a variable number of arguments - each argument should be an ANSII code.\n\n#### Example\n```python\nimport pixy\n\n# Bright blue using ANSII code 94 supported by some terminals\nprint(pixy.pring("Hello World", pixy.EscapeSequence(94)))\n``` \n## Examples\n\n### Blinking Text\n```python\nimport pixy\n\nprint(pixy.pring("Hello World", pixy.decorators.slow_blink))\n```\n\n### Concatenation\n```python\nimport pixy\n\n# "Hello" in red.\ns1 = pixy.pring("Hello", pixy.foreground.red)\n\n# " World" in blue.\ns2 = pixy.pring(" World", pixy.foreground.blue)\n\n# Concatenate the strings, add a white background and make\n# them bold.\ns3 = pixy.pring(s1 + s2, pixy.background.white + pixy.decorators.bold)\n\nprint(s3)\n```\n\n### Colour shades\n```python\nimport pixy\n\nfor i in range(0, 0xFF, 3): \n    colour = pixy.TrueColour(i, 0, 0, background=True)\n    print(pixy.pring(" ", colour), end="")\n\nprint()\n\nfor i in range(0, 0xFF, 3): \n    colour = pixy.TrueColour(0, i, 0, background=True)\n    print(pixy.pring(" ", colour), end="")\n\nprint()\n\nfor i in range(0, 0xFF, 3): \n    colour = pixy.TrueColour(0, 0, i, background=True)\n    print(pixy.pring(" ", colour), end="")\n\nprint()\n```\n\n### Colour gradient\n```python\nimport pixy\n\ndef gradient(colour_a, colour_b):\n    \n    output = []\n\n    for i in range(0, 100, 2): \n\n        p = i / 100 \n\n        output.append((\n            int(colour_a[0] + p * (colour_b[0] - colour_a[0])),\n            int(colour_a[1] + p * (colour_b[1] - colour_a[1])),\n            int(colour_a[2] + p * (colour_b[2] - colour_a[2]))\n        ))\n\n    return output\n\nred_to_green = gradient((255, 0, 0), (0, 255, 0)) \ngreen_to_blue = gradient((0, 255, 0), (0, 0, 255))\nblue_to_red = gradient((0, 0, 255), (255, 0, 0)) \n\nfor colour in red_to_green:\n    print(pixy.pring(" ", pixy.TrueColour(*colour, background=True)), end="")\n\nprint()\n\nfor colour in green_to_blue:\n    print(pixy.pring(" ", pixy.TrueColour(*colour, background=True)), end="")\n\nprint()\n\nfor colour in blue_to_red:\n    print(pixy.pring(" ", pixy.TrueColour(*colour, background=True)), end="")\n\nprint()\n```\n\nLicense\n-------\n\npixy is licensed under the [MIT license](https://github.com/kiancross/pixy/blob/master/LICENSE).\n',
    'author': 'Kian Cross',
    'author_email': 'kian@kiancross.co.uk',
    'maintainer': 'Kian Cross',
    'maintainer_email': 'kian@kiancross.co.uk',
    'url': 'https://github.com/kiancross/pixy',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



#
# Copyright (c) 2020 Kian Cross
#

from pixy.escape_sequence import EscapeSequence

bold = EscapeSequence(1)
faint = EscapeSequence(2)
italic = EscapeSequence(3)
underline = EscapeSequence(4)
slow_blink = EscapeSequence(5)
rapid_blink = EscapeSequence(6)
invert = EscapeSequence(7)
conceal = EscapeSequence(8)
strike = EscapeSequence(9)
fraktur = EscapeSequence(20)
double_underline = EscapeSequence(21)
framed = EscapeSequence(51)
encircled = EscapeSequence(52)
overlined = EscapeSequence(53)
